@@ -2,11 +2,12 @@
 #include <RcppArmadillo.h>
 #include "make_swaps_helper.h"
 #include "sw_mh_helper.h"
+#include "constraint_calc_helper.h"
 
 using namespace Rcpp;
 
-// [[Rcpp::export()]]
-List rsg (List adj_list,
+// [[Rcpp::export]]
+List rsg(List adj_list,
                NumericVector population,
                int Ndistrict,
                double target_pop,
@@ -31,11 +32,11 @@ List rsg (List adj_list,
 
 	// member_dvec tracks which district each precinct belongs to
 	// Always of length Nprecinct
-	IntegerVector member_dvec(Nprecinct);	
+	IntegerVector member_dvec(Nprecinct);
 
 	// List of vectors, where each vector is an Integervector of precincts in that district
 	// This eventually shrinks to length Ndistrict
-	List member_plist(Nprecinct);	
+	List member_plist(Nprecinct);
 
 	// Step 1: Each precinct represents a single district
 	int Ndist_current = Nprecinct;
@@ -47,7 +48,7 @@ List rsg (List adj_list,
 	}
 
 	while(Ndist_current > Ndistrict){
-	  
+
 	  //STEP 2A: Select one of the N (Ndist_current) districts and denote it as district i
 	  i_dist_vec = runif(1, 0, 1000000000);
 	  i_dist = fmod(i_dist_vec(0), Ndist_current);
@@ -63,7 +64,7 @@ List rsg (List adj_list,
 	    }
 	  } //end if
 	  idist_pneighbors = setdiff( idist_pneighbors, idist_pmembers );
-	  
+
 	  // Generate list of adjacent districts
 	  // Get each neighboring precinct's district membership, and take it's unique vector
 	  // Note: union_(x,x) gives the unique elements of x
@@ -72,12 +73,12 @@ List rsg (List adj_list,
 	    idist_dneighbors[i] = member_dvec[ idist_pneighbors[i] ];
 	  }
 	  idist_dneighbors = unique(idist_dneighbors);
-	  
+
 	  //STEP 2B: Select one of district i's bordering districts at random and denote it as district j
 	  j_dist_vec = runif(1, 0, 1000000000);
 	  j_dist = idist_dneighbors[fmod(j_dist_vec(0), idist_dneighbors.size())];
 	  jdist_pmembers = IntegerVector(member_plist[j_dist]);
-	  
+
 	  //STEP 2C: Merge district i with district j to form a new district
 	  // We merge j into i --- so ensure that i < j and swap if necessary
 	  if( i_dist > j_dist){
@@ -85,7 +86,7 @@ List rsg (List adj_list,
 	    i_dist = j_dist;
 	    j_dist = i;
 	  }
-	  
+
 	  // Loop over all precincts, update member_dvec 2 ways
 	  // Every jdist becomes idist
 	  // jdist gets wiped, so decrement every district counter above it
@@ -93,30 +94,30 @@ List rsg (List adj_list,
 	    if(member_dvec[i] == j_dist) member_dvec[i] = i_dist;
 	    if(member_dvec[i] > j_dist) member_dvec[i] = member_dvec[i] - 1;
 	  }
-	  
+
 	  // Update member_plist
 	  idist_newmembers = union_(idist_pmembers, jdist_pmembers) ;
 	  member_plist[i_dist] = idist_newmembers;	// merge member lists in idist
 	  member_plist.erase(j_dist);				//delete jdist's membership list
-	  
+
 	  Ndist_current = Ndist_current - 1;
 
 	}	// end while(Ndist_current > Ndistrict)
 
 
-	// Calculate district population	
+	// Calculate district population
 	// Only total this once --- we iteratively only add/subtract a single precinct from this after each switch
-	for(i=0; i< Nprecinct; i++)	district_pop[ member_dvec[i] ] += population[i]; 
+	for(i=0; i< Nprecinct; i++)	district_pop[ member_dvec[i] ] += population[i];
 
 
 	iter=0;
 	while( (max(district_pop) > maxpop) | (min(district_pop) < minpop) ){
-	  
+
 	  // WHILE POPULATION CONSTRAINT NOT MET
-	  
+
 	  // STEP 3A: Identify the most populated district and denote it as district i
 	  i_dist = which_max(district_pop);
-  
+
 	  // If more than one district has max population, break the tie randomly
 	  j=0;
 	  for(i=0; i<Ndistrict; i++){
@@ -127,22 +128,22 @@ List rsg (List adj_list,
 	  }
 	  maxpop_dist_vec = runif(1, 0, 1000000000);
 	  i_dist = maxpop_dist[fmod(maxpop_dist_vec(0), j)];
-	  
+
 	  // STEP 3B: Select a random precinct in that district, and denote it as precinct p
 	  idist_pmembers = IntegerVector(member_plist[i_dist]);
 	  p_index_vec = runif(1, 0, 1000000000);
 	  p_index = fmod(p_index_vec(0), idist_pmembers.size());
 	  p = idist_pmembers[p_index];
-	  
-	  
+
+
 	  // STEP 3C: If precinct p can be reassigned from district i to a new district
 	  // without violating contiguity of either this district or district i, reassign it
 	  p_neighbors = IntegerVector(adj_list[p]);
 	  p_neighbors_size = p_neighbors.size();
 	  d_neighbors = IntegerVector(p_neighbors_size);
 	  for(i=0; i< p_neighbors_size; i++)	d_neighbors[i] = member_dvec[p_neighbors[i]];
-	  
-	  
+
+
 	  // Generate list of adjacent districts
 	  IntegerVector j_candidates;
 	  j_candidates = unique(d_neighbors);
@@ -152,12 +153,12 @@ List rsg (List adj_list,
 		if( j_candidates[i] == i_dist ) j_candidates.erase(i);
 	  }
 	  j_candidates_size = j_candidates.size();
-	  
+
 	  // If there is a valid move, choose a valid j_dist to move to
 	  if( j_candidates_size > 0){
 
 	    j_candidates_vec = runif(1, 0, 1000000000);
-	    j_dist = j_candidates[fmod(j_candidates_vec(0), j_candidates_size)]; 
+	    j_dist = j_candidates[fmod(j_candidates_vec(0), j_candidates_size)];
 
 		//Check check that move from i_dist to j_dist is valid
 		NumericVector member_new_dvec;
@@ -171,9 +172,9 @@ List rsg (List adj_list,
 
 			district_pop[i_dist] -= population[p];
 	    	district_pop[j_dist] += population[p];
-	    
+
 	    	member_dvec[p] = j_dist;
-	    
+
 	    	//Assumed here that if idist_members.size==1 (a single precinct district), it will never be the largest district
 	    	idist_members = IntegerVector(member_plist[i_dist]);
 		    for(i= (idist_members.size() - 1); i > -1 ; i--){
@@ -181,13 +182,13 @@ List rsg (List adj_list,
 		    }
 		    jdist_members = IntegerVector(member_plist[j_dist]);
 		    jdist_members.push_back(p);
-	    
+
 		    member_plist[i_dist] = idist_members;
 		    member_plist[j_dist] = jdist_members;
 			}
-	    
+
 	  } // end if(i_valid==1){
-	  
+
 	  iter++;
 	  if(iter > maxiter){
 	    List result;
@@ -196,7 +197,7 @@ List rsg (List adj_list,
 	    result["district_pop"] = NumericVector::get_na();
 	    return(result);
 	  }
-	  
+
 	}  // end while( max(district_pop) > maxpop){
 
 	// Output is:
